import 'dart:async';

import 'package:idb_shim/idb_browser.dart';
import 'package:idb_shim/idb_client.dart';
import 'package:lori_sdk/data/database/interfaces/authResponseDao.dart';
import 'package:lori_sdk/data/models/authResponse.dart';
import 'package:lori_sdk/utils/lori_utils.dart';

const int kVersion1 = 1;

class WebAuthResponseDao implements AuthResponseDao {
  final IdbFactory idbFactory = idbFactoryBrowser;
  Database db;

  static final String authResponseStoreName = 'auth_responses';

  Future open() async {
    if (db == null) {
      db = await idbFactory.open('auth_response.db', version: kVersion1,
          onUpgradeNeeded: (event) {
        var db = event.database;
        db.createObjectStore(authResponseStoreName, autoIncrement: true);
      });
    }
  }

  Future<void> insertOrUpdate(AuthResponse response) async {
    var storedResponse = await get(response.responseType);
    if (storedResponse == null) {
      return insert(response);
    } else {
      return update(response);
    }
  }

  Future<void> insert(AuthResponse response) async {
    await open();
    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;
    await authResponseWritableTxn.add(response.toMap());
    return response;
  }

  Future<void> update(AuthResponse response) async {
    await open();
    if (response.refreshToken == null) response.refreshToken = '';
    response.responseCreatedDate = DateTime.now().millisecondsSinceEpoch;
    return authResponseWritableTxn.put(response.toMap(), response.tokenType);
  }

  Future<AuthResponse> get(int responseType) async {
    await open();
    List<AuthResponse> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = authResponseReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          final auth = cursorToAuthResponse(cursor);
          if (auth.responseType == responseType) {
            list.add(auth);
          }
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    return list.length > 0 ? list.first : null;
  }

  Future<int> delete(int responseType) async {
    await open();
    return authResponseWritableTxn.delete(responseType);
  }

  Future<void> deleteAll() async {
    await open();
    await authResponseWritableTxn
        .openKeyCursor(autoAdvance: true)
        .listen((cursor) {
      cursor.delete();
    }).asFuture();
    return;
  }

  Future close() async => db.close();

  // Helper to convert AuthResponse from CURSOR to a MAP
  AuthResponse cursorToAuthResponse(CursorWithValue cursor) {
    AuthResponse response = AuthResponse('', '', 0, '');
    var snapshot = asMap(cursor.value);
    if (snapshot != null) {
      response.fromMap(
        snapshot,
      );
    }
    return response;
  }

  // Accessors to read and write from the database
  ObjectStore get authResponseWritableTxn {
    var txn = db.transaction(authResponseStoreName, idbModeReadWrite);
    var store = txn.objectStore(authResponseStoreName);
    return store;
  }

  ObjectStore get authResponseReadableTxn {
    var txn = db.transaction(authResponseStoreName, idbModeReadOnly);
    var store = txn.objectStore(authResponseStoreName);
    return store;
  }
}

AuthResponseDao getAuthResponseDao() => WebAuthResponseDao();
