import 'dart:async';

import 'package:idb_shim/idb_browser.dart';
import 'package:idb_shim/idb_client.dart';
import 'package:lori_sdk/data/database/interfaces/cartDao.dart';
import 'package:lori_sdk/data/models/cart/cart.dart';
import 'package:lori_sdk/data/models/singleton/cartSingleton.dart';
import 'package:lori_sdk/utils/lori_utils.dart';

const int kVersion1 = 1;

class WebCartDao implements CartDao {
  final IdbFactory idbFactory = idbFactoryBrowser;
  Database db;

  static final String cartStoreName = 'carts';

  Future open() async {
    if (db == null) {
      db = await idbFactory.open('carts.db', version: kVersion1,
          onUpgradeNeeded: (event) {
        var db = event.database;
        db.createObjectStore(cartStoreName, autoIncrement: true);
      });
    }
  }

  Future<Cart> get() async {
    await open();
    List<Cart> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = cartReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          final cart = cursorToCart(cursor);
          list.add(cart);
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    return list.length > 0 ? list.first : null;
  }

  Future<void> insertOrUpdate(Cart cart) async {
    var storedResponse = await get();
    if (storedResponse == null) {
      return insert(cart);
    } else {
      return update(cart);
    }
  }

  Future<void> insert(Cart cart) async {
    await open();
    await cartWritableTxn.add(cart.toMap());
    CartSingleton().cartChanged(cart);
    return cart;
  }

  Future<void> update(Cart cart) async {
    await open();
    CartSingleton().cartChanged(cart);
    return cartWritableTxn.put(cart.toMap(), cart.id);
  }

  Future<void> deleteAll() async {
    await open();
    await cartWritableTxn.openKeyCursor(autoAdvance: true).listen((cursor) {
      cursor.delete();
    }).asFuture();
    return;
  }

  // Helper to convert AuthResponse from CURSOR to a MAP
  Cart cursorToCart(CursorWithValue cursor) {
    Cart cart = Cart();
    var snapshot = asMap(cursor.value);
    if (snapshot != null) {
      cart.fromMap(
        snapshot,
      );
    }
    return cart;
  }

  // Accessors to read and write from the database
  ObjectStore get cartWritableTxn {
    var txn = db.transaction(cartStoreName, idbModeReadWrite);
    var store = txn.objectStore(cartStoreName);
    return store;
  }

  ObjectStore get cartReadableTxn {
    var txn = db.transaction(cartStoreName, idbModeReadOnly);
    var store = txn.objectStore(cartStoreName);
    return store;
  }
}

CartDao getCartDao() => WebCartDao();
