import 'dart:async';

import 'package:idb_shim/idb_browser.dart';
import 'package:idb_shim/idb_client.dart';
import 'package:lori_sdk/data/database/interfaces/notificationDao.dart';
import 'package:lori_sdk/data/models/notification.dart';
import 'package:lori_sdk/data/models/singleton/notificationSingleton.dart';
import 'package:lori_sdk/utils/lori_utils.dart';

const int kVersion1 = 1;

class WebNotificationDao implements NotificationDao {
  final IdbFactory idbFactory = idbFactoryBrowser;
  Database db;

  static final String notificationsStoreName = 'notifications';

  Future open() async {
    if (db == null) {
      db = await idbFactory.open('notifications.db', version: kVersion1,
          onUpgradeNeeded: (event) {
        var db = event.database;
        db.createObjectStore(notificationsStoreName, autoIncrement: true);
      });
    }
  }

  Future<void> insertOrUpdate(Notification notification) async {
    // Get by text
    if (notification.id == null || notification.id == 0) {
      var notif = await getByText(notification.title, notification.message);
      if (notif == null)
        return insert(notification);
      else {
        notif.title = notification.title;
        notif.message = notification.message;
        return updateByText(notif);
      }
    } else {
      // Get by id
      var notif = await getById(notification.id);
      if (notif == null)
        return insert(notification);
      else
        return update(notification);
    }
  }

  Future<void> insert(Notification notification) async {
    await open();
    var count = await get(false);
    notification.id = count.length + 1;
    await notificationWritableTxn.add(notification.toMap());
    return NotificationSingleton().notificationNumberChanged(count.length + 1);
  }

  Future<void> update(Notification notification) async {
    await open();
    final map = notification.toMap();
    map.remove('id');
    await notificationWritableTxn.put(map, notification.id);
    var count = await get(false);
    return NotificationSingleton().notificationNumberChanged(count.length);
  }

  Future<void> updateByText(Notification notification) async {
    await open();
    final map = notification.toMap();
    map.remove('id');
    await notificationWritableTxn.put(map, notification.id);
    var count = await get(false);
    return NotificationSingleton().notificationNumberChanged(count.length);
  }

  Future<List<Notification>> get(bool isRead) async {
    await open();
    List<Notification> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = notificationReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          final notification = cursorToNotification(cursor);
          if ((isRead && notification.isRead) ||
              (!isRead && !notification.isRead)) {
            list.add(notification);
          }
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    return list;
  }

  Future<Notification> getByText(String title, String body) async {
    await open();
    List<Notification> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = notificationReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          final notification = cursorToNotification(cursor);
          if (notification.title == title && notification.message == body) {
            list.add(notification);
          }
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    return list.length > 0 ? list.first : null;
  }

  Future<Notification> getById(int id) async {
    await open();
    List<Notification> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = notificationReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          final notification = cursorToNotification(cursor);
          if (notification.id == id) {
            list.add(notification);
          }
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    return list.length > 0 ? list.first : null;
  }

  Future<void> deleteAll() async {
    await open();
    await notificationWritableTxn
        .openKeyCursor(autoAdvance: true)
        .listen((cursor) {
      cursor.delete();
    }).asFuture();
    return;
  }

  // Helper to convert AuthResponse from CURSOR to a MAP
  Notification cursorToNotification(CursorWithValue cursor) {
    Notification notification = Notification.empty();
    var snapshot = asMap(cursor.value);
    if (snapshot != null) {
      notification.fromMap(
        snapshot,
      );
    }
    return notification;
  }

  // Accessors to read and write from the database
  ObjectStore get notificationWritableTxn {
    var txn = db.transaction(notificationsStoreName, idbModeReadWrite);
    var store = txn.objectStore(notificationsStoreName);
    return store;
  }

  ObjectStore get notificationReadableTxn {
    var txn = db.transaction(notificationsStoreName, idbModeReadOnly);
    var store = txn.objectStore(notificationsStoreName);
    return store;
  }
}

NotificationDao getNotificationDao() => WebNotificationDao();
