import 'dart:async';

import 'package:idb_shim/idb_browser.dart';
import 'package:idb_shim/idb_client.dart';
import 'package:lori_sdk/data/database/interfaces/productDao.dart';
import 'package:lori_sdk/data/models/product/product.dart';
import 'package:lori_sdk/data/models/singleton/productSingleton.dart';
import 'package:lori_sdk/utils/lori_utils.dart';

const int kVersion1 = 1;

class WebProductDao implements ProductDao {
  final IdbFactory idbFactory = idbFactoryBrowser;
  Database db;

  static final String productsStoreName = 'products';

  Future open() async {
    if (db == null) {
      db = await idbFactory.open('products.db', version: kVersion1,
          onUpgradeNeeded: (event) {
        var db = event.database;
        db.createObjectStore(productsStoreName, autoIncrement: true);
      });
    }
  }

  Future<void> insertOrUpdate(Product product) async {
    var storedProducts = await get();
    Product readProduct;
    if (storedProducts != null) {
      readProduct =
          storedProducts.firstWhere((sp) => sp.id == product.id, orElse: () {
        return null;
      });
    }
    if (readProduct == null) {
      return await insert(product);
    } else {
      return await update(product);
    }
  }

  Future<void> insert(Product product) async {
    product.lastTimestamp = DateTime.now().millisecondsSinceEpoch;
    await open();
    await productWritableTxn.add(product.toMap());
    return ProductSingleton().productsChanged(await get());
  }

  Future<void> update(Product product) async {
    product.lastTimestamp = DateTime.now().millisecondsSinceEpoch;
    await open();
    await productWritableTxn.put(product.toMap(), product.id);
    return ProductSingleton().productsChanged(await get());
  }

  Future<List<Product>> get() async {
    await open();
    List<Product> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = productReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          list.add(cursorToProduct(cursor));
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    list.sort((p1, p2) => p2.lastTimestamp.compareTo(p1.lastTimestamp));
    return list.length == 0 ? null : list;
  }

  Future<void> delete() async {
    await open();
    await productWritableTxn.openKeyCursor(autoAdvance: true).listen((cursor) {
      cursor.delete();
    }).asFuture();
    return;
  }

  // Helper to convert AuthResponse from CURSOR to a MAP
  Product cursorToProduct(CursorWithValue cursor) {
    Product product = Product();
    var snapshot = asMap(cursor.value);
    if (snapshot != null) {
      product.fromMap(
        snapshot,
      );
    }
    return product;
  }

  // Accessors to read and write from the database
  ObjectStore get productWritableTxn {
    var txn = db.transaction(productsStoreName, idbModeReadWrite);
    var store = txn.objectStore(productsStoreName);
    return store;
  }

  ObjectStore get productReadableTxn {
    var txn = db.transaction(productsStoreName, idbModeReadOnly);
    var store = txn.objectStore(productsStoreName);
    return store;
  }
}

ProductDao getProductDao() => WebProductDao();
