import 'dart:async';

import 'package:idb_shim/idb_browser.dart';
import 'package:idb_shim/idb_client.dart';
import 'package:lori_sdk/data/database/interfaces/userDao.dart';
import 'package:lori_sdk/data/models/singleton/userSingleton.dart';
import 'package:lori_sdk/data/models/user.dart';
import 'package:lori_sdk/utils/lori_utils.dart';

const int kVersion1 = 1;

class WebUserDao implements UserDao {
  final IdbFactory idbFactory = idbFactoryBrowser;
  Database db;
  User loggedUser;

  static final String userStoreName = 'users';

  Future open() async {
    if (db == null) {
      db = await idbFactory.open('users.db', version: kVersion1,
          onUpgradeNeeded: (event) {
        var db = event.database;
        db.createObjectStore(userStoreName, autoIncrement: true);
      });
    }
  }

  Future<void> insertOrUpdate(User user) async {
    var storedUser = await get();
    if (storedUser == null) {
      return await insert(user);
    } else {
      return await update(user);
    }
  }

  Future<void> insert(User user) async {
    await open();
    await userWritableTxn.add(user.toMap());
    return UserSingleton().userChanged(user);
  }

  Future<void> update(User user) async {
    await open();
    await userWritableTxn.put(user.toMap(), user.id);
    return UserSingleton().userChanged(user);
  }

  Future<User> get() async {
    await open();
    List<User> list = [];
    // ignore: cancel_subscriptions
    StreamSubscription subscription;
    subscription = userReadableTxn
        .openCursor(direction: idbDirectionPrev, autoAdvance: true)
        .listen((cursor) {
      try {
        var map = asMap<String, dynamic>(cursor.value);
        if (map != null) {
          final user = cursorToUser(cursor);
          list.add(user);
        }
      } catch (e) {
        // Error converting the cursor to map
      }
    });
    await subscription.asFuture();
    return list.length > 0 ? list.first : null;
  }

  Future<void> delete() async {
    await open();
    await userWritableTxn.openKeyCursor(autoAdvance: true).listen((cursor) {
      cursor.delete();
    }).asFuture();
    return;
  }

  // Helper to convert AuthResponse from CURSOR to a MAP
  User cursorToUser(CursorWithValue cursor) {
    User user = User();
    var snapshot = asMap(cursor.value);
    if (snapshot != null) {
      user.fromMap(
        snapshot,
      );
    }
    return user;
  }

  // Accessors to read and write from the database
  ObjectStore get userWritableTxn {
    var txn = db.transaction(userStoreName, idbModeReadWrite);
    var store = txn.objectStore(userStoreName);
    return store;
  }

  ObjectStore get userReadableTxn {
    var txn = db.transaction(userStoreName, idbModeReadOnly);
    var store = txn.objectStore(userStoreName);
    return store;
  }
}

UserDao getUserDao() => WebUserDao();
